@@ -41,6 +41,8 @@ typedef struct _exsltFuncResultPreComp exsltFuncResultPreComp;
 struct _exsltFuncResultPreComp {
     xsltElemPreComp comp;
     xmlXPathCompExprPtr select;
+    xmlNsPtr *nsList;
+    int nsNr;
 };
 
 /* Used for callback function in exsltInitFunc */
@@ -184,13 +186,13 @@ exsltFuncShutdown (xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED,
  * @style: an XSLT stylesheet
  * @URI: the namespace URI for the extension
  *
- * Allocates the styleshet data for EXSLT - Function
+ * Allocates the stylesheet data for EXSLT - Function
  *
  * Returns the allocated data
  */
 static xmlHashTablePtr
-exsltFuncStyleInit (xsltStylesheetPtr style,
-		    const xmlChar *URI) {
+exsltFuncStyleInit (xsltStylesheetPtr style ATTRIBUTE_UNUSED,
+		    const xmlChar *URI ATTRIBUTE_UNUSED) {
     return xmlHashCreate(1);
 }
 
@@ -247,6 +249,8 @@ exsltFreeFuncResultPreComp (exsltFuncResultPreComp *comp) {
 
     if (comp->select != NULL)
 	xmlXPathFreeCompExpr (comp->select);
+    if (comp->nsList != NULL)
+        xmlFree(comp->nsList);
     xmlFree(comp);
 }
 
@@ -262,7 +266,8 @@ exsltFuncFunctionFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     xmlXPathObjectPtr obj, oldResult, ret;
     exsltFuncData *data;
     exsltFuncFunctionData *func;
-    xmlNodePtr paramNode, oldInsert, fake;
+    xmlNodePtr paramNode, oldInsert, fake, content = NULL;
+    int oldBase;
     xsltStackElemPtr params = NULL, param;
     xsltTransformContextPtr tctxt = xsltXPathGetTransformContext(ctxt);
     int i;
@@ -289,8 +294,10 @@ exsltFuncFunctionFunction (xmlXPathParserContextPtr ctxt, int nargs) {
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
     }
-    if (func->content != NULL)
+    if (func->content != NULL) {
 	paramNode = func->content->prev;
+	content = func->content;
+    }
     else
 	paramNode = NULL;
     if ((paramNode == NULL) && (func->nargs != 0)) {
@@ -299,14 +306,13 @@ exsltFuncFunctionFunction (xmlXPathParserContextPtr ctxt, int nargs) {
 			 "param == NULL\n");
 	return;
     }
-    /* defaulted params */
-    for (i = func->nargs; (i > nargs) && (paramNode != NULL); i--) {
-	param = xsltParseStylesheetCallerParam (tctxt, paramNode);
-	param->next = params;
-	params = param;
-	paramNode = paramNode->prev;
-    }
+
     /* set params */
+    for (i = func->nargs; (i > nargs) && (paramNode != NULL); i--) {
+	paramNode = paramNode->prev;
+	if (content != NULL)
+	    content = content->prev;
+    }
     while ((i-- > 0) && (paramNode != NULL)) {
 	obj = valuePop(ctxt);
 	/* FIXME: this is a bit hackish */
@@ -327,9 +333,16 @@ exsltFuncFunctionFunction (xmlXPathParserContextPtr ctxt, int nargs) {
 			 (const xmlChar *)"fake", NULL);
     oldInsert = tctxt->insert;
     tctxt->insert = fake;
+    /* 
+     * In order to give the function variables a new 'scope' we
+     * change varsBase in the context.
+     */
+    oldBase = tctxt->varsBase;
+    tctxt->varsBase = tctxt->varsNr;
     xsltApplyOneTemplate (tctxt, xmlXPathGetContextNode(ctxt),
-			  func->content, NULL, params);
+			  content, NULL, params);
     tctxt->insert = oldInsert;
+    tctxt->varsBase = oldBase;	/* restore original scope */
     if (params != NULL)
 	xsltFreeStackElemList(params);
 
@@ -442,7 +455,7 @@ static xsltElemPreCompPtr
 exsltFuncResultComp (xsltStylesheetPtr style, xmlNodePtr inst,
 		     xsltTransformFunction function) {
     xmlNodePtr test;
-    xmlChar *select;
+    xmlChar *sel;
     exsltFuncResultPreComp *ret;
 
     /*
@@ -512,12 +525,21 @@ exsltFuncResultComp (xsltStylesheetPtr style, xmlNodePtr inst,
     /*
      * Precompute the select attribute
      */
-    select = xmlGetNsProp(inst, (const xmlChar *) "select", NULL);
-    if (select != NULL) {
-	ret->select = xmlXPathCompile (select);
-	xmlFree(select);
+    sel = xmlGetNsProp(inst, (const xmlChar *) "select", NULL);
+    if (sel != NULL) {
+	ret->select = xmlXPathCompile (sel);
+	xmlFree(sel);
+    }
+    /*
+     * Precompute the namespace list
+     */
+    ret->nsList = xmlGetNsList(inst->doc, inst);
+    if (ret->nsList != NULL) {
+        int i = 0;
+        while (ret->nsList[i] != NULL)
+	    i++;
+	ret->nsNr = i;
     }
-
     return ((xsltElemPreCompPtr) ret);
 }
 
@@ -527,6 +549,8 @@ exsltFuncResultElem (xsltTransformContextPtr ctxt,
 		     exsltFuncResultPreComp *comp) {
     exsltFuncData *data;
     xmlXPathObjectPtr ret;
+    xmlNsPtr *oldNsList;
+    int oldNsNr;
 
     /* It is an error if instantiating the content of the
      * func:function element results in the instantiation of more than
@@ -560,7 +584,13 @@ exsltFuncResultElem (xsltTransformContextPtr ctxt,
 	    data->error = 1;
 	    return;
 	}
+	oldNsList = ctxt->xpathCtxt->namespaces;
+	oldNsNr = ctxt->xpathCtxt->nsNr;
+	ctxt->xpathCtxt->namespaces = comp->nsList;
+	ctxt->xpathCtxt->nsNr = comp->nsNr;
 	ret = xmlXPathCompiledEval(comp->select, ctxt->xpathCtxt);
+	ctxt->xpathCtxt->nsNr = oldNsNr;
+	ctxt->xpathCtxt->namespaces = oldNsList;
 	if (ret == NULL) {
 	    xsltGenericError(xsltGenericErrorContext,
 			     "exsltFuncResultElem: ret == NULL\n");
