@@ -1921,13 +1921,17 @@ xsltGetProfileInformation(xsltTransformContextPtr ctxt)
  */
 xmlXPathCompExprPtr
 xsltXPathCompile(xsltStylesheetPtr style, const xmlChar *str) {
-    xmlXPathContext xctxt;
+    xmlXPathContextPtr xctxt;
     xmlXPathCompExprPtr ret;
 
-    memset(&xctxt, 0, sizeof(xctxt));
     if (style != NULL)
-	xctxt.dict = style->dict;
-    ret = xmlXPathCtxtCompile(&xctxt, str);
+	xctxt = xmlXPathNewContext(style->doc);
+    else
+	xctxt = xmlXPathNewContext(NULL);
+    if (style != NULL)
+	xctxt->dict = style->dict;
+    ret = xmlXPathCtxtCompile(xctxt, str);
+    xmlXPathFreeContext(xctxt);
     /*
      * TODO: there is a lot of optimizations which should be possible
      *       like variable slot precomputations, function precomputations, etc.
@@ -1935,6 +1939,7 @@ xsltXPathCompile(xsltStylesheetPtr style, const xmlChar *str) {
      
     return(ret);
 }
+
 /************************************************************************
  * 									*
  * 		Hooks for the debugger					*
